/*
 *	spFrameP.h
 */

#ifndef __SPFRAMEP_H
#define __SPFRAMEP_H

#include <sp/spPrimitiveP.h>
#include <sp/spFrame.h>
#if defined(GTK)
#include <sp/spFrame_Gtk.h>
#elif defined(_WIN32) && !defined(USE_MOTIF)
#include <sp/spFrame_Win.h>
#elif defined(ANDROID)
#include <sp/spFrame_Android.h>
#elif defined(COCOA)
#include <sp/spFrame_Cocoa.h>
#elif defined(MACOS)
#include <sp/spFrame_Mac.h>
#else
#include <sp/spFrame_Xm.h>
#endif

#ifdef __cplusplus
extern "C" {
#endif

typedef struct _spFramePart {
    spFrameArchPart arch_part;
    spComponent menu_bar;
    spComponent tool_bar;
    spComponent status_bar;
    spComponent task_tray_menu;
    spComponent default_button;
    spComponent cancel_button;
    spComponent parent_window;		/* if not null, the window is sub-window */
    spTopLevel toplevel;
    void *system_window;		/* if not null, arch-depend parent window is set directly */
    spTimerList timer;
    
    char *icon_name;			/* icon name */
    char *task_tray_icon;		/* icon name for task tray */
    char *task_tray_tip;		/* tips for task tray */
    spWindowType window_type;		/* window type */
    spPopupStyle popup_style;		/* window popup style */
    spCloseStyle close_style;		/* window close style */
    spBool resize_flag;
    spBool iconfy_flag;
    spBool task_tray_visible;		/* is task tray visible? */
    spBool help_button_visible;
    spBool internal_flag;
    spBool simplify_flag;
    spBool topmost_flag;
    spBool position_set_flag;
    spDialogBoxButtonType button_type;
} spFramePart;

typedef struct _spFrameRec {
    spObjectPart object;
    spComponentPart component;
    spPrimitivePart primitive;
    spFramePart frame;
} spFrameRec;
    
typedef struct _spFrameClassPart {
    int dummy;
} spFrameClassPart;

typedef struct _spFrameClassRec {
    spObjectClassPart object;
    spComponentClassPart component;
    spPrimitiveClassPart primitive;
    spFrameClassPart frame;
} spFrameClassRec;

struct _spTimerList {
    spComponent frame;
    spTimerCallbackFunc call_func;
    void *call_data;
    long interval_ms;

    spTimerId timer_id;
#if defined(USE_MOTIF)
    XtIntervalId xt_timer_id;
#elif defined(COCOA)
#elif defined(MACOS) && !TARGET_API_MAC_CARBON
    unsigned long last_tick;
#endif

    spBool kill_flag;

    spTimerList prev;
    spTimerList next;
};
    
#if defined(MACOS)
#pragma import on
#endif

extern spFrameClassRec SpFrameClassRec;

#define SpFramePart(comp) (((spFrame)(comp))->frame)
#define SpFrameArch(comp) (((spFrame)(comp))->frame.arch_part)
#define SpParentFramePart(comp) (((spFrame)SpGetParent(comp))->frame)
#define SpParentFrameArch(comp) (SpParentFramePart(comp).arch_part)
#define SpGetFrameClass(comp) ((spFrameClass)((comp)->object.object_class))

extern void spFramePartInit(spObject object);
extern void spFramePartInitArch(spComponent component);
extern void spFramePartFree(spObject object);
extern void spFramePartFreeArch(spComponent component);
extern spBool spFrameCreate(spObject object);
extern spBool spFrameDestroy(spObject object);
extern spBool spFrameSetParams(spObject object);
extern void spFrameCreateArch(spComponent component);
extern void spFrameDestroyArch(spComponent component);
extern void spFrameSetParamsArch(spComponent component);
extern void *spGetArchFrame(spComponent component);
extern spBool spShowTaskTrayArch(spComponent window, spBool flag);
extern void spPopupFrameArch(spComponent frame, spBool move_cursor);
extern void spPopdownFrameArch(spComponent frame);
extern void spRaiseFrameArch(spComponent window);
extern void spLowerFrameArch(spComponent window);
extern spBool spSetFramePositionArch(spComponent window, int x, int y);
extern spBool spGetFramePositionArch(spComponent window, int *x, int *y);
extern spBool spGetFrameModifierKeyMaskArch(spComponent window, spModifierMask *mask);
extern spTimerList spAddTimerList(spComponent frame, long interval_ms,
				  spTimerCallbackFunc call_func, void *call_data);
extern void spRemoveTimerList(spComponent frame, spTimerList timer);
extern spTimerList spFindTimerList(spComponent frame, spTimerId timer_id);
extern spTimerId spSetTimerArch(spTimerList timer);
extern spBool spKillTimerArch(spTimerList timer);
extern spBool spAddDropCallbackArch(spComponent component, spDropCallbackFunc call_func,
				    void *call_data);

#if defined(MACOS)
#pragma import off
#endif

#ifdef __cplusplus
}  /* Close scope of 'extern "C"' declaration */
#endif

#endif /* __SPFRAMEP_H */
