/*
 *        spPrimitiveP.h
 */

#ifndef __SPPRIMITIVEP_H
#define __SPPRIMITIVEP_H

#include <sp/spComponentP.h>
#include <sp/spPrimitive.h>
#if defined(GTK)
#include <sp/spPrimitive_Gtk.h>
#elif defined(_WIN32) && !defined(USE_MOTIF)
#include <sp/spPrimitive_Win.h>
#elif defined(ANDROID)
#include <sp/spPrimitive_Android.h>
#elif defined(COCOA)
#include <sp/spPrimitive_Cocoa.h>
#elif defined(MACOS)
#include <sp/spPrimitive_Mac.h>
#else
#include <sp/spPrimitive_Xm.h>
#endif

#ifdef __cplusplus
extern "C" {
#endif

#if defined(SP_USE_DEFAULT_CALLBACK_STRUCT)
struct _spCallback {
    spComponent component;
    spCallbackFunc call_func;
    void *call_data;
    spBool propagate;
    
    spCallbackType call_type;
};
    
extern spBool spPostMessageDefault(spComponent component, spCallbackType call_type);
#endif

typedef struct _spEventCallbackList *spEventCallbackList;
    
struct _spEventCallbackList {
    spEventCallbackFunc call_func;

    spEventCallbackList prev;
    spEventCallbackList next;
};
    
typedef struct _spPrimitivePart {
    /* private */
    spPrimitiveArchPart arch_part; /* architecture dependent part */
    int index;                  /* index used for some component */
    spCallbackFunc draw_func;   /* function pointer for drawable */
    void *draw_data;            /* data for drawable's callback */
    long num_callback;          /* number of callback functions */
    long num_callback_buffer;   /* size of callback buffer */
    spCallback *callbacks;      /* callback functions */

    spEventCallbackList event_func_list;
    void *pad;
} spPrimitivePart;
    
typedef struct _spPrimitiveRec {
    spObjectPart object;
    spComponentPart component;
    spPrimitivePart primitive;
} spPrimitiveRec;
    
typedef struct _spPrimitiveClassPart {
    int dummy;
} spPrimitiveClassPart;

typedef struct _spPrimitiveClassRec {
    spObjectClassPart object;
    spComponentClassPart component;
    spPrimitiveClassPart primitive;
} spPrimitiveClassRec;

#if defined(MACOS)
#pragma import on
#endif

extern spPrimitiveClassRec SpPrimitiveClassRec;

#define SpPrimitivePart(comp) (((spPrimitive)(comp))->primitive)
#define SpPrimitiveArch(comp) (((spPrimitive)(comp))->primitive.arch_part)
#define SpParentPrimitivePart(comp) (((spPrimitive)SpGetParent(comp))->primitive)
#define SpParentPrimitiveArch(comp) (SpParentPrimitivePart(comp).arch_part)
#define SpGetPrimitiveClass(comp) ((spPrimitiveClass)((comp)->object.object_class))

extern void spInitCallbackArch(spCallback *callback);
extern void spPrimitivePartInitArch(spComponent component);
extern void spPrimitiveDestroyArch(spComponent component);
extern void spPrimitiveMapArch(spComponent component);
extern void spPrimitiveUnmapArch(spComponent component);
extern void spPrimitiveScaleDefaultSizeArch(spComponent component);
extern spBool spPrimitiveSetSizeArch(spComponent component, int width, int height);
extern spBool spPrimitiveGetSizeArch(spComponent component, int *width, int *height);
extern spBool spPrimitiveGetClientSizeArch(spComponent component, int *width, int *height);
extern spBool spPrimitiveSetSensitiveArch(spComponent component, spBool flag);
extern spBool spPrimitiveAddCallbackArch(spComponent component, spBool propagate, spCallbackType call_type,
                                         spCallbackFunc call_func, void *call_data);
extern spBool spPrimitiveRemoveCallbackArch(spComponent component, spCallbackType call_type,
                                            spCallbackFunc call_func, void *call_data);
extern spCallbackReason spPrimitiveGetCallbackReasonArch(spComponent component);
extern spBool spPrimitiveGetCallbackMousePositionArch(spComponent component, int *x, int *y);
extern spBool spPrimitiveGetCallbackWheelValueArch(spComponent component, double *delta_x, double *delta_y);
extern spBool spPrimitiveGetCallbackKeySymArch(spComponent component, spKeySym *key_sym);
extern int spPrimitiveGetCallbackKeyStringArch(spComponent component,
                                               char *buf, int buf_size, spBool *overflow);

extern void spAllocCallbacks(spComponent component);
extern void spFreeCallbacks(spComponent component);
extern void spPrimitivePartInit(spObject object);
extern void spPrimitivePartFree(spObject object);
extern spBool spPrimitiveCreate(spObject object);
extern spBool spPrimitiveDestroy(spObject object);
extern void spPrimitiveMap(spComponent component);
extern void spPrimitiveUnmap(spComponent component);
extern spBool spPrimitiveSetSize(spComponent component, int width, int height);
extern spBool spPrimitiveGetSize(spComponent component, int *width, int *height);
extern spBool spPrimitiveGetClientSize(spComponent component, int *width, int *height);
extern spBool spPrimitiveSetSensitive(spComponent component, spBool flag);
extern spBool spPrimitiveAddCallback(spComponent component, spBool propagate, spCallbackType call_type,
                                     spCallbackFunc call_func, void *call_data);
extern spBool spPrimitiveRemoveCallback(spComponent component, spCallbackType call_type,
                                        spCallbackFunc call_func, void *call_data);
extern spCallbackReason spPrimitiveCallbackFunc(spComponent component);
extern spBool spPrimitiveGetCallbackMousePosition(spComponent component, int *x, int *y);
extern spBool spPrimitiveGetCallbackKeySym(spComponent component, spKeySym *key_sym);
extern int spPrimitiveGetCallbackKeyString(spComponent component,
                                           char *buf, int buf_size, spBool *overflow);
extern void spPrimitiveSetDefaultSize(spComponent component);
extern spBool spShowToolTipArch(spComponent component);
extern spBool spPrepareEventCallbackArch(spComponent component);
extern spBool spUnprepareEventCallbackArch(spComponent component);
extern spBool spPostUserEventArch(spComponent component, spEventType event_type, void *call_data);

extern spBool spAddEventCallbackList(spEventCallbackList *first_list, spEventCallbackFunc call_func);
extern spBool spRemoveEventCallbackList(spEventCallbackList *first_list, spEventCallbackFunc call_func);
extern spBool spDestroyEventCallbackList(spEventCallbackList *first_list);
extern spBool spCallEventCallbackList(spEventCallbackList *first_list, spComponent component, spEventType event_type, void *call_data);
extern spBool spExistsPrimitiveEventCallback(spComponent primitive);
extern spBool spPrimitiveCallEventCallback(spComponent primitive, spEventType event_type, void *call_data);
    
#if defined(SP_USE_DEFAULT_USER_EVENT)
extern spBool spAllocUserEventDefault(spUserEventDefault *first_event, long count);
extern spUserEventDefault spAppendUserEventDefault(spUserEventDefault *first_event,
                                                   spComponent component, spEventType event_type, void *call_data);
extern spBool spDestroyUserEventDefault(spUserEventDefault *first_event, spUserEventDefault event);
extern spBool spDestroyAllUserEventDefault(spUserEventDefault *first_event);
extern spUserEventDefault spGetLastUserEventDefault(spUserEventDefault *first_event);
extern long spGetNumUserEventDefault(spUserEventDefault *first_event);
extern spBool spProcessUserEventDefault(spUserEventDefault *first_event, spUserEventDefault event);
extern long spProcessPendingUserEventDefault(spUserEventDefault *first_event, long end_count);
#endif
    
extern void *spGetPrimitiveHandleArch(spComponent component);

#if defined(MACOS)
#pragma import off
#endif

#ifdef __cplusplus
}  /* Close scope of 'extern "C"' declaration */
#endif

#endif /* __SPPRIMITIVEP_H */
