/*
 *	spTopLevelP.h
 */

#ifndef __SPTOPLEVELP_H
#define __SPTOPLEVELP_H

#include <sp/spObjectP.h>
#include <sp/spGraphics.h>
#include <sp/spTopLevel.h>
#if defined(GTK)
#include <sp/spTopLevel_Gtk.h>
#elif defined(_WIN32) && !defined(USE_MOTIF)
#include <sp/spTopLevel_Win.h>
#elif defined(ANDROID)
#include <sp/spTopLevel_Android.h>
#elif defined(COCOA)
#include <sp/spTopLevel_Cocoa.h>
#elif defined(MACOS)
#include <sp/spTopLevel_Mac.h>
#else
#include <sp/spTopLevel_Xm.h>
#endif

#ifdef __cplusplus
extern "C" {
#endif

typedef struct _spTopLevelPart {
    spTopLevelArchPart arch_part;	/* architecture dependent part */
    int *argcp;
    char ***argvp;
    spOptions options;
    spGraphics graphics;
    spGraphics last_graphics;
    
    spBool alt_ctrl_swap;
    spBool exit_prompt;
    spBool close_prompt;
    spBool modify_prompt;
    spBool thread_safe;
    spBool use_window_menu;
    char *language;
    char *icon_name;
    char *information;
    char *unique_id;
    char *domain_id;
    char *help_path;

    spTopLevelCallbackFunc call_func;	/* function pointer */
    void *call_data;			/* data for callback */

    unsigned long permissions;
    unsigned long accepted_permissions;
    unsigned long disabled_permissions; /* denied and disabled forever */
    unsigned long asked_permissions;
} spTopLevelPart;

typedef struct _spTopLevelRec {
    spObjectPart object;
    spTopLevelPart toplevel;
} spTopLevelRec;
    
typedef struct _spTopLevelClassPart {
    int dummy;
} spTopLevelClassPart;

typedef struct _spTopLevelClassRec {
    spObjectClassPart object;
    spTopLevelClassPart toplevel;
} spTopLevelClassRec;

#if defined(MACOS)
#pragma import on
#endif

extern spTopLevelClassRec SpTopLevelClassRec;

#define SpTopLevelPart(obj) (((spTopLevel)obj)->toplevel)
#define SpTopLevelArch(obj) (((spTopLevel)obj)->toplevel.arch_part)
#define SpGetTopLevelClass(obj) ((spTopLevelClass)((obj)->object.object_class))

extern void spTopLevelPartInit(spObject object);
extern void spTopLevelPartInitArch(spTopLevel toplevel);
extern void spTopLevelPartFree(spObject object);
extern void spTopLevelPartFreeArch(spTopLevel toplevel);
extern spBool spTopLevelCreate(spObject object);
extern void spTopLevelCreateArch(spTopLevel toplevel);
extern spBool spTopLevelSetParams(spObject object);
extern void spTopLevelSetParamsArch(spTopLevel toplevel);
extern spBool spTopLevelDestroy(spObject object);
extern void spTopLevelDestroyArch(spTopLevel toplevel);
extern int spWaitNextEventArch(spTopLevel toplevel, spBool lock_flag);
extern int spDispatchEventArch(spTopLevel toplevel, spBool lock_flag);
extern int spMainLoopArch(spTopLevel toplevel);
extern void spQuitArch(int status);
extern void spExitArch(int status);
extern spBool spThreadEnterArch(spTopLevel toplevel);
extern spBool spThreadLeaveArch(spTopLevel toplevel);
extern spBool spSuspendMainLockArch(spTopLevel toplevel);
extern spBool spResumeMainLockArch(spTopLevel toplevel);
extern long spWaitThreadLoopDefault(spTopLevel toplevel, void *handle, spBool dispatch_flag);
extern spBool spWaitThreadPostDummyEventArch(spTopLevel toplevel);
extern long spWaitThreadLoopArch(spTopLevel toplevel, void *handle);
    
extern spBool spIsPostEventSupportedArch(spTopLevel toplevel);

#if defined(ANDROID)
extern spBool spDemandPermissionRequestArch(spTopLevel toplevel, unsigned long permissions);
extern spBool spIsPermissionRequestRequiredArch(spTopLevel toplevel, unsigned long permissions);
#endif

extern spBool spGetHelpTopDirDefault(char *dir, int dir_size, char separator);
extern spBool spGetHelpFileDefault(const char *dir, const char *path, char *file, int file_size, char separator);

#if defined(MACOS)
#pragma import off
#endif

#ifdef __cplusplus
}  /* Close scope of 'extern "C"' declaration */
#endif

#endif /* __SPTOPLEVELP_H */
